import abc
import typing

import QuantConnect.Optimizer
import QuantConnect.Optimizer.Objectives
import QuantConnect.Optimizer.Parameters
import QuantConnect.Optimizer.Strategies
import QuantConnect.Packets
import System
import System.Collections.Concurrent
import System.Collections.Generic

QuantConnect_Optimizer__EventContainer_Callable = typing.TypeVar("QuantConnect_Optimizer__EventContainer_Callable")
QuantConnect_Optimizer__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Optimizer__EventContainer_ReturnType")


class OptimizationResult(System.Object):
    """Defines the result of Lean compute job"""

    Initial: QuantConnect.Optimizer.OptimizationResult = ...
    """Corresponds to initial result to drive the optimization strategy"""

    @property
    def BacktestId(self) -> str:
        """The backtest id that generated this result"""
        ...

    @property
    def Id(self) -> int:
        """Parameter set Id"""
        ...

    @property
    def JsonBacktestResult(self) -> str:
        """Json Backtest result"""
        ...

    @property
    def ParameterSet(self) -> QuantConnect.Optimizer.Parameters.ParameterSet:
        """The parameter set at which the result was achieved"""
        ...

    def __init__(self, jsonBacktestResult: str, parameterSet: QuantConnect.Optimizer.Parameters.ParameterSet, backtestId: str) -> None:
        """
        Create an instance of OptimizationResult
        
        :param jsonBacktestResult: Optimization target value for this backtest
        :param parameterSet: Parameter set used in compute job
        :param backtestId: The backtest id that generated this result
        """
        ...


class OptimizationNodePacket(QuantConnect.Packets.Packet):
    """Provide a packet type containing information on the optimization compute job."""

    @property
    def UserId(self) -> int:
        """User Id placing request"""
        ...

    @UserId.setter
    def UserId(self, value: int):
        """User Id placing request"""
        ...

    @property
    def UserToken(self) -> str:
        ...

    @UserToken.setter
    def UserToken(self, value: str):
        ...

    @property
    def ProjectId(self) -> int:
        """Project Id of the request"""
        ...

    @ProjectId.setter
    def ProjectId(self, value: int):
        """Project Id of the request"""
        ...

    @property
    def CompileId(self) -> str:
        """Unique compile id of this optimization"""
        ...

    @CompileId.setter
    def CompileId(self, value: str):
        """Unique compile id of this optimization"""
        ...

    @property
    def OptimizationId(self) -> str:
        """The unique optimization Id of the request"""
        ...

    @OptimizationId.setter
    def OptimizationId(self, value: str):
        """The unique optimization Id of the request"""
        ...

    @property
    def OrganizationId(self) -> str:
        """Organization Id of the request"""
        ...

    @OrganizationId.setter
    def OrganizationId(self, value: str):
        """Organization Id of the request"""
        ...

    @property
    def MaximumConcurrentBacktests(self) -> int:
        """Limit for the amount of concurrent backtests being run"""
        ...

    @MaximumConcurrentBacktests.setter
    def MaximumConcurrentBacktests(self, value: int):
        """Limit for the amount of concurrent backtests being run"""
        ...

    @property
    def OptimizationStrategy(self) -> str:
        """Optimization strategy name"""
        ...

    @OptimizationStrategy.setter
    def OptimizationStrategy(self, value: str):
        """Optimization strategy name"""
        ...

    @property
    def Criterion(self) -> QuantConnect.Optimizer.Objectives.Target:
        """Objective settings"""
        ...

    @Criterion.setter
    def Criterion(self, value: QuantConnect.Optimizer.Objectives.Target):
        """Objective settings"""
        ...

    @property
    def Constraints(self) -> System.Collections.Generic.IReadOnlyList[QuantConnect.Optimizer.Objectives.Constraint]:
        """Optimization constraints"""
        ...

    @Constraints.setter
    def Constraints(self, value: System.Collections.Generic.IReadOnlyList[QuantConnect.Optimizer.Objectives.Constraint]):
        """Optimization constraints"""
        ...

    @property
    def OptimizationParameters(self) -> System.Collections.Generic.HashSet[QuantConnect.Optimizer.Parameters.OptimizationParameter]:
        """The user optimization parameters"""
        ...

    @OptimizationParameters.setter
    def OptimizationParameters(self, value: System.Collections.Generic.HashSet[QuantConnect.Optimizer.Parameters.OptimizationParameter]):
        """The user optimization parameters"""
        ...

    @property
    def OptimizationStrategySettings(self) -> QuantConnect.Optimizer.Strategies.OptimizationStrategySettings:
        """The user optimization parameters"""
        ...

    @OptimizationStrategySettings.setter
    def OptimizationStrategySettings(self, value: QuantConnect.Optimizer.Strategies.OptimizationStrategySettings):
        """The user optimization parameters"""
        ...

    @typing.overload
    def __init__(self) -> None:
        """Creates a new instance"""
        ...

    @typing.overload
    def __init__(self, packetType: QuantConnect.Packets.PacketType) -> None:
        """
        Creates a new instance
        
        This method is protected.
        """
        ...


class OptimizationStatus(System.Enum):
    """The different optimization status"""

    New = 0
    """Just created and not running optimization"""

    Aborted = 1
    """We failed or we were aborted"""

    Running = 2
    """We are running"""

    Completed = 3
    """Optimization job has completed"""


class LeanOptimizer(System.Object, System.IDisposable, metaclass=abc.ABCMeta):
    """Base Lean optimizer class in charge of handling an optimization job packet"""

    @property
    def CompletedBacktests(self) -> int:
        """
        The total completed backtests count
        
        This property is protected.
        """
        ...

    @property
    def Status(self) -> int:
        """
        The current optimization status
        
        This property contains the int value of a member of the QuantConnect.Optimizer.OptimizationStatus enum.
        
        This property is protected.
        """
        ...

    @Status.setter
    def Status(self, value: int):
        """
        The current optimization status
        
        This property contains the int value of a member of the QuantConnect.Optimizer.OptimizationStatus enum.
        
        This property is protected.
        """
        ...

    @property
    def OptimizationTarget(self) -> QuantConnect.Optimizer.Objectives.Target:
        """
        The optimization target
        
        This field is protected.
        """
        ...

    @property
    def RunningParameterSetForBacktest(self) -> System.Collections.Concurrent.ConcurrentDictionary[str, QuantConnect.Optimizer.Parameters.ParameterSet]:
        """
        Collection holding ParameterSet for each backtest id we are waiting to finish
        
        This field is protected.
        """
        ...

    @property
    def PendingParameterSet(self) -> System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Optimizer.Parameters.ParameterSet]:
        """
        Collection holding ParameterSet for each backtest id we are waiting to launch
        
        This field is protected.
        """
        ...

    @property
    def Strategy(self) -> QuantConnect.Optimizer.Strategies.IOptimizationStrategy:
        """
        The optimization strategy being used
        
        This field is protected.
        """
        ...

    @property
    def NodePacket(self) -> QuantConnect.Optimizer.OptimizationNodePacket:
        """
        The optimization packet
        
        This field is protected.
        """
        ...

    @property
    def Disposed(self) -> bool:
        """
        Indicates whether optimizer was disposed
        
        This property is protected.
        """
        ...

    @property
    def Ended(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Optimizer.OptimizationResult], None], None]:
        """Event triggered when the optimization work ended"""
        ...

    @Ended.setter
    def Ended(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Optimizer.OptimizationResult], None], None]):
        """Event triggered when the optimization work ended"""
        ...

    def __init__(self, nodePacket: QuantConnect.Optimizer.OptimizationNodePacket) -> None:
        """
        Creates a new instance
        
        This method is protected.
        
        :param nodePacket: The optimization node packet to handle
        """
        ...

    def AbortLean(self, backtestId: str) -> None:
        """
        Handles stopping Lean process
        
        This method is protected.
        
        :param backtestId: Specified backtest id
        """
        ...

    def Dispose(self) -> None:
        """Disposes of any resources"""
        ...

    def GetCurrentEstimate(self) -> int:
        """Returns the current optimization status and strategy estimates"""
        ...

    def GetLogDetails(self) -> str:
        """
        Helper method to have pretty more informative logs
        
        This method is protected.
        """
        ...

    def GetRuntimeStatistics(self) -> System.Collections.Generic.Dictionary[str, str]:
        """Get the current runtime statistics"""
        ...

    def NewResult(self, jsonBacktestResult: str, backtestId: str) -> None:
        """
        Handles a new backtest json result matching a requested backtest id
        
        This method is protected.
        
        :param jsonBacktestResult: The backtest json result
        :param backtestId: The associated backtest id
        """
        ...

    def RunLean(self, parameterSet: QuantConnect.Optimizer.Parameters.ParameterSet) -> str:
        """
        Handles starting Lean for a given parameter set
        
        This method is protected.
        
        :param parameterSet: The parameter set for the backtest to run
        :returns: The new unique backtest id.
        """
        ...

    def SendUpdate(self) -> None:
        """
        Sends an update of the current optimization status to the user
        
        This method is protected.
        """
        ...

    def SetOptimizationStatus(self, optimizationStatus: QuantConnect.Optimizer.OptimizationStatus) -> None:
        """
        Sets the current optimization status
        
        This method is protected.
        
        :param optimizationStatus: The new optimization status
        """
        ...

    def Start(self) -> None:
        """Starts the optimization"""
        ...

    def TriggerOnEndEvent(self) -> None:
        """
        Triggers the optimization job end event
        
        This method is protected.
        """
        ...


class _EventContainer(typing.Generic[QuantConnect_Optimizer__EventContainer_Callable, QuantConnect_Optimizer__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Optimizer__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Optimizer__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Optimizer__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





import abc
import datetime
import typing
import warnings

import System
import System.Collections
import System.Collections.Generic
import System.IO
import System.Net
import System.Net.Sockets
import System.Runtime.Serialization
import System.Security
import System.Security.Authentication.ExtendedProtection


class WebUtility(System.Object):
    """This class has no documentation."""

    @staticmethod
    @typing.overload
    def HtmlDecode(value: str) -> str:
        ...

    @staticmethod
    @typing.overload
    def HtmlDecode(value: str, output: System.IO.TextWriter) -> None:
        ...

    @staticmethod
    @typing.overload
    def HtmlEncode(value: str) -> str:
        ...

    @staticmethod
    @typing.overload
    def HtmlEncode(value: str, output: System.IO.TextWriter) -> None:
        ...

    @staticmethod
    def UrlDecode(encodedValue: str) -> str:
        ...

    @staticmethod
    def UrlDecodeToBytes(encodedValue: typing.List[int], offset: int, count: int) -> typing.List[int]:
        ...

    @staticmethod
    def UrlEncode(value: str) -> str:
        ...

    @staticmethod
    def UrlEncodeToBytes(value: typing.List[int], offset: int, count: int) -> typing.List[int]:
        ...


class Cookie(System.Object):
    """This class has no documentation."""

    MaxSupportedVersion: int = 1

    MaxSupportedVersionString: str = "1"

    SeparatorLiteral: str = "; "

    EqualsLiteral: str = ...

    QuotesLiteral: str = ...

    SpecialAttributeLiteral: str = "$"

    PortSplitDelimiters: typing.List[str] = ...

    ReservedToName: typing.List[str] = ...

    ReservedToValue: typing.List[str] = ...

    @property
    def IsQuotedVersion(self) -> bool:
        ...

    @IsQuotedVersion.setter
    def IsQuotedVersion(self, value: bool):
        ...

    @property
    def IsQuotedDomain(self) -> bool:
        ...

    @IsQuotedDomain.setter
    def IsQuotedDomain(self, value: bool):
        ...

    @property
    def Comment(self) -> str:
        ...

    @Comment.setter
    def Comment(self, value: str):
        ...

    @property
    def CommentUri(self) -> System.Uri:
        ...

    @CommentUri.setter
    def CommentUri(self, value: System.Uri):
        ...

    @property
    def HttpOnly(self) -> bool:
        ...

    @HttpOnly.setter
    def HttpOnly(self, value: bool):
        ...

    @property
    def Discard(self) -> bool:
        ...

    @Discard.setter
    def Discard(self, value: bool):
        ...

    @property
    def Domain(self) -> str:
        ...

    @Domain.setter
    def Domain(self, value: str):
        ...

    @property
    def DomainImplicit(self) -> bool:
        ...

    @DomainImplicit.setter
    def DomainImplicit(self, value: bool):
        ...

    @property
    def Expired(self) -> bool:
        ...

    @Expired.setter
    def Expired(self, value: bool):
        ...

    @property
    def Expires(self) -> datetime.datetime:
        ...

    @Expires.setter
    def Expires(self, value: datetime.datetime):
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...

    @property
    def Path(self) -> str:
        ...

    @Path.setter
    def Path(self, value: str):
        ...

    @property
    def Plain(self) -> bool:
        ...

    @property
    def Port(self) -> str:
        ...

    @Port.setter
    def Port(self, value: str):
        ...

    @property
    def PortList(self) -> typing.List[int]:
        ...

    @property
    def Secure(self) -> bool:
        ...

    @Secure.setter
    def Secure(self, value: bool):
        ...

    @property
    def TimeStamp(self) -> datetime.datetime:
        ...

    @property
    def Value(self) -> str:
        ...

    @Value.setter
    def Value(self, value: str):
        ...

    @property
    def Variant(self) -> int:
        """This property contains the int value of a member of the System.Net.CookieVariant enum."""
        ...

    @property
    def DomainKey(self) -> str:
        ...

    @property
    def Version(self) -> int:
        ...

    @Version.setter
    def Version(self, value: int):
        ...

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, name: str, value: str) -> None:
        ...

    @typing.overload
    def __init__(self, name: str, value: str, path: str) -> None:
        ...

    @typing.overload
    def __init__(self, name: str, value: str, path: str, domain: str) -> None:
        ...

    def Equals(self, comparand: typing.Any) -> bool:
        ...

    def GetHashCode(self) -> int:
        ...

    def ToString(self) -> str:
        ...


class CookieCollection(System.Object, System.Collections.Generic.ICollection[System.Net.Cookie], System.Collections.Generic.IReadOnlyCollection[System.Net.Cookie], System.Collections.ICollection, typing.Iterable[System.Net.Cookie]):
    """This class has no documentation."""

    @property
    def IsReadOnly(self) -> bool:
        ...

    @property
    def Count(self) -> int:
        ...

    @property
    def IsSynchronized(self) -> bool:
        ...

    @property
    def SyncRoot(self) -> System.Object:
        ...

    @property
    def IsOtherVersionSeen(self) -> bool:
        ...

    @typing.overload
    def __getitem__(self, index: int) -> System.Net.Cookie:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> System.Net.Cookie:
        ...

    def __init__(self) -> None:
        ...

    @typing.overload
    def Add(self, cookie: System.Net.Cookie) -> None:
        ...

    @typing.overload
    def Add(self, cookies: System.Net.CookieCollection) -> None:
        ...

    def Clear(self) -> None:
        ...

    def Contains(self, cookie: System.Net.Cookie) -> bool:
        ...

    @typing.overload
    def CopyTo(self, array: System.Array, index: int) -> None:
        ...

    @typing.overload
    def CopyTo(self, array: typing.List[System.Net.Cookie], index: int) -> None:
        ...

    @typing.overload
    def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System.Net.Cookie]:
        ...

    @typing.overload
    def GetEnumerator(self) -> System.Collections.IEnumerator:
        ...

    def Remove(self, cookie: System.Net.Cookie) -> bool:
        ...


class CookieContainer(System.Object):
    """This class has no documentation."""

    DefaultCookieLimit: int = 300

    DefaultPerDomainCookieLimit: int = 20

    DefaultCookieLengthLimit: int = 4096

    @property
    def Capacity(self) -> int:
        ...

    @Capacity.setter
    def Capacity(self, value: int):
        ...

    @property
    def Count(self) -> int:
        ...

    @property
    def MaxCookieSize(self) -> int:
        ...

    @MaxCookieSize.setter
    def MaxCookieSize(self, value: int):
        ...

    @property
    def PerDomainCapacity(self) -> int:
        ...

    @PerDomainCapacity.setter
    def PerDomainCapacity(self, value: int):
        ...

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, capacity: int) -> None:
        ...

    @typing.overload
    def __init__(self, capacity: int, perDomainCapacity: int, maxCookieSize: int) -> None:
        ...

    @typing.overload
    def Add(self, cookie: System.Net.Cookie) -> None:
        ...

    @typing.overload
    def Add(self, cookies: System.Net.CookieCollection) -> None:
        ...

    @typing.overload
    def Add(self, uri: System.Uri, cookie: System.Net.Cookie) -> None:
        ...

    @typing.overload
    def Add(self, uri: System.Uri, cookies: System.Net.CookieCollection) -> None:
        ...

    def GetAllCookies(self) -> System.Net.CookieCollection:
        """
        Gets a CookieCollection that contains all of the Cookie instances in the container.
        
        :returns: A CookieCollection that contains all of the Cookie instances in the container.
        """
        ...

    def GetCookieHeader(self, uri: System.Uri) -> str:
        ...

    def GetCookies(self, uri: System.Uri) -> System.Net.CookieCollection:
        ...

    def SetCookies(self, uri: System.Uri, cookieHeader: str) -> None:
        ...


class PathList(System.Object):
    """This class has no documentation."""

    @property
    def Count(self) -> int:
        ...

    @property
    def Values(self) -> System.Collections.ICollection:
        ...

    @property
    def List(self) -> System.Collections.SortedList:
        ...

    @property
    def SyncRoot(self) -> System.Object:
        ...


class HttpVersion(System.Object):
    """This class has no documentation."""

    Unknown: System.Version = ...
    """Defines a Version instance that indicates an unknown version of HTTP."""

    Version10: System.Version = ...
    """Defines a Version instance for HTTP 1.0."""

    Version11: System.Version = ...
    """Defines a Version instance for HTTP 1.1."""

    Version20: System.Version = ...
    """Defines a Version instance for HTTP 2.0."""

    Version30: System.Version = ...
    """Defines a Version instance for HTTP 3.0."""


class CookieException(System.FormatException, System.Runtime.Serialization.ISerializable):
    """This class has no documentation."""

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, serializationInfo: System.Runtime.Serialization.SerializationInfo, streamingContext: System.Runtime.Serialization.StreamingContext) -> None:
        """This method is protected."""
        ...

    @typing.overload
    def GetObjectData(self, serializationInfo: System.Runtime.Serialization.SerializationInfo, streamingContext: System.Runtime.Serialization.StreamingContext) -> None:
        ...

    @typing.overload
    def GetObjectData(self, serializationInfo: System.Runtime.Serialization.SerializationInfo, streamingContext: System.Runtime.Serialization.StreamingContext) -> None:
        ...


class DecompressionMethods(System.Enum):
    """This class has no documentation."""

    # Cannot convert to Python: None = 0

    GZip = ...

    Deflate = ...

    Brotli = ...

    All = ...


class IPAddress(System.Object):
    """This class has no documentation."""

    Any: System.Net.IPAddress = ...

    Loopback: System.Net.IPAddress = ...

    Broadcast: System.Net.IPAddress = ...

    # Cannot convert to Python: None: System.Net.IPAddress = ...

    LoopbackMaskHostOrder: int = ...

    IPv6Any: System.Net.IPAddress = ...

    IPv6Loopback: System.Net.IPAddress = ...

    IPv6None: System.Net.IPAddress = ...

    NumberOfLabels: int = ...

    @property
    def AddressFamily(self) -> int:
        """This property contains the int value of a member of the System.Net.Sockets.AddressFamily enum."""
        ...

    @property
    def ScopeId(self) -> int:
        ...

    @ScopeId.setter
    def ScopeId(self, value: int):
        ...

    @property
    def IsIPv6Multicast(self) -> bool:
        ...

    @property
    def IsIPv6LinkLocal(self) -> bool:
        ...

    @property
    def IsIPv6SiteLocal(self) -> bool:
        ...

    @property
    def IsIPv6Teredo(self) -> bool:
        ...

    @property
    def IsIPv6UniqueLocal(self) -> bool:
        """Gets whether the address is an IPv6 Unique Local address."""
        ...

    @property
    def IsIPv4MappedToIPv6(self) -> bool:
        ...

    @property
    def Address(self) -> int:
        """IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead."""
        warnings.warn("IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead.", DeprecationWarning)

    @Address.setter
    def Address(self, value: int):
        """IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead."""
        warnings.warn("IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead.", DeprecationWarning)

    @typing.overload
    def __init__(self, newAddress: int) -> None:
        ...

    @typing.overload
    def __init__(self, address: typing.List[int], scopeid: int) -> None:
        ...

    @typing.overload
    def __init__(self, address: System.ReadOnlySpan[int], scopeid: int) -> None:
        ...

    @typing.overload
    def __init__(self, address: typing.List[int]) -> None:
        ...

    @typing.overload
    def __init__(self, address: System.ReadOnlySpan[int]) -> None:
        ...

    def Equals(self, comparand: typing.Any) -> bool:
        """Compares two IP addresses."""
        ...

    def GetAddressBytes(self) -> typing.List[int]:
        ...

    def GetHashCode(self) -> int:
        ...

    @staticmethod
    @typing.overload
    def HostToNetworkOrder(host: int) -> int:
        ...

    @staticmethod
    @typing.overload
    def HostToNetworkOrder(host: int) -> int:
        ...

    @staticmethod
    @typing.overload
    def HostToNetworkOrder(host: int) -> int:
        ...

    @staticmethod
    def IsLoopback(address: System.Net.IPAddress) -> bool:
        ...

    def MapToIPv4(self) -> System.Net.IPAddress:
        ...

    def MapToIPv6(self) -> System.Net.IPAddress:
        ...

    @staticmethod
    @typing.overload
    def NetworkToHostOrder(network: int) -> int:
        ...

    @staticmethod
    @typing.overload
    def NetworkToHostOrder(network: int) -> int:
        ...

    @staticmethod
    @typing.overload
    def NetworkToHostOrder(network: int) -> int:
        ...

    @staticmethod
    @typing.overload
    def Parse(ipString: str) -> System.Net.IPAddress:
        ...

    @staticmethod
    @typing.overload
    def Parse(ipSpan: System.ReadOnlySpan[str]) -> System.Net.IPAddress:
        ...

    def ToString(self) -> str:
        ...

    def TryFormat(self, destination: System.Span[str], charsWritten: typing.Optional[int]) -> typing.Union[bool, int]:
        ...

    @staticmethod
    @typing.overload
    def TryParse(ipString: str, address: typing.Optional[System.Net.IPAddress]) -> typing.Union[bool, System.Net.IPAddress]:
        ...

    @staticmethod
    @typing.overload
    def TryParse(ipSpan: System.ReadOnlySpan[str], address: typing.Optional[System.Net.IPAddress]) -> typing.Union[bool, System.Net.IPAddress]:
        ...

    def TryWriteBytes(self, destination: System.Span[int], bytesWritten: typing.Optional[int]) -> typing.Union[bool, int]:
        ...


class EndPoint(System.Object, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def AddressFamily(self) -> int:
        """This property contains the int value of a member of the System.Net.Sockets.AddressFamily enum."""
        ...

    def Create(self, socketAddress: System.Net.SocketAddress) -> System.Net.EndPoint:
        ...

    def Serialize(self) -> System.Net.SocketAddress:
        ...


class NetworkCredential(System.Object, System.Net.ICredentials, System.Net.ICredentialsByHost):
    """This class has no documentation."""

    @property
    def UserName(self) -> str:
        ...

    @UserName.setter
    def UserName(self, value: str):
        ...

    @property
    def Password(self) -> str:
        ...

    @Password.setter
    def Password(self, value: str):
        ...

    @property
    def SecurePassword(self) -> System.Security.SecureString:
        ...

    @SecurePassword.setter
    def SecurePassword(self, value: System.Security.SecureString):
        ...

    @property
    def Domain(self) -> str:
        ...

    @Domain.setter
    def Domain(self, value: str):
        ...

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, userName: str, password: str) -> None:
        ...

    @typing.overload
    def __init__(self, userName: str, password: str, domain: str) -> None:
        ...

    @typing.overload
    def __init__(self, userName: str, password: System.Security.SecureString) -> None:
        ...

    @typing.overload
    def __init__(self, userName: str, password: System.Security.SecureString, domain: str) -> None:
        ...

    @typing.overload
    def GetCredential(self, uri: System.Uri, authenticationType: str) -> System.Net.NetworkCredential:
        ...

    @typing.overload
    def GetCredential(self, host: str, port: int, authenticationType: str) -> System.Net.NetworkCredential:
        ...


class ICredentials(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def GetCredential(self, uri: System.Uri, authType: str) -> System.Net.NetworkCredential:
        ...


class ICredentialsByHost(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def GetCredential(self, host: str, port: int, authenticationType: str) -> System.Net.NetworkCredential:
        ...


class CredentialCache(System.Object, System.Net.ICredentials, System.Net.ICredentialsByHost, System.Collections.IEnumerable):
    """This class has no documentation."""

    DefaultCredentials: System.Net.ICredentials

    DefaultNetworkCredentials: System.Net.NetworkCredential

    def __init__(self) -> None:
        ...

    @typing.overload
    def Add(self, uriPrefix: System.Uri, authType: str, cred: System.Net.NetworkCredential) -> None:
        ...

    @typing.overload
    def Add(self, host: str, port: int, authenticationType: str, credential: System.Net.NetworkCredential) -> None:
        ...

    @typing.overload
    def GetCredential(self, uriPrefix: System.Uri, authType: str) -> System.Net.NetworkCredential:
        ...

    @typing.overload
    def GetCredential(self, host: str, port: int, authenticationType: str) -> System.Net.NetworkCredential:
        ...

    def GetEnumerator(self) -> System.Collections.IEnumerator:
        ...

    @typing.overload
    def Remove(self, uriPrefix: System.Uri, authType: str) -> None:
        ...

    @typing.overload
    def Remove(self, host: str, port: int, authenticationType: str) -> None:
        ...


class AuthenticationSchemes(System.Enum):
    """This class has no documentation."""

    # Cannot convert to Python: None = ...

    Digest = ...

    Negotiate = ...

    Ntlm = ...

    Basic = ...

    Anonymous = ...

    IntegratedWindowsAuthentication = ...


class IWebProxy(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def Credentials(self) -> System.Net.ICredentials:
        ...

    @Credentials.setter
    @abc.abstractmethod
    def Credentials(self, value: System.Net.ICredentials):
        ...

    def GetProxy(self, destination: System.Uri) -> System.Uri:
        ...

    def IsBypassed(self, host: System.Uri) -> bool:
        ...


class CookieVariant(System.Enum):
    """This class has no documentation."""

    Unknown = 0

    Plain = 1

    Rfc2109 = 2

    Rfc2965 = 3

    Default = ...


class DnsEndPoint(System.Net.EndPoint):
    """This class has no documentation."""

    @property
    def Host(self) -> str:
        ...

    @property
    def AddressFamily(self) -> int:
        """This property contains the int value of a member of the System.Net.Sockets.AddressFamily enum."""
        ...

    @property
    def Port(self) -> int:
        ...

    @typing.overload
    def __init__(self, host: str, port: int) -> None:
        ...

    @typing.overload
    def __init__(self, host: str, port: int, addressFamily: System.Net.Sockets.AddressFamily) -> None:
        ...

    def Equals(self, comparand: typing.Any) -> bool:
        ...

    def GetHashCode(self) -> int:
        ...

    def ToString(self) -> str:
        ...


class IPEndPoint(System.Net.EndPoint):
    """Provides an IP address."""

    MinPort: int = ...
    """Specifies the minimum acceptable value for the System.Net.IPEndPoint.Port property."""

    MaxPort: int = ...
    """Specifies the maximum acceptable value for the System.Net.IPEndPoint.Port property."""

    @property
    def AddressFamily(self) -> int:
        """This property contains the int value of a member of the System.Net.Sockets.AddressFamily enum."""
        ...

    @property
    def Address(self) -> System.Net.IPAddress:
        """Gets or sets the IP address."""
        ...

    @Address.setter
    def Address(self, value: System.Net.IPAddress):
        """Gets or sets the IP address."""
        ...

    @property
    def Port(self) -> int:
        """Gets or sets the port."""
        ...

    @Port.setter
    def Port(self, value: int):
        """Gets or sets the port."""
        ...

    @typing.overload
    def __init__(self, address: int, port: int) -> None:
        """Creates a new instance of the IPEndPoint class with the specified address and port."""
        ...

    @typing.overload
    def __init__(self, address: System.Net.IPAddress, port: int) -> None:
        """Creates a new instance of the IPEndPoint class with the specified address and port."""
        ...

    def Create(self, socketAddress: System.Net.SocketAddress) -> System.Net.EndPoint:
        ...

    def Equals(self, comparand: typing.Any) -> bool:
        ...

    def GetHashCode(self) -> int:
        ...

    @staticmethod
    @typing.overload
    def Parse(s: str) -> System.Net.IPEndPoint:
        ...

    @staticmethod
    @typing.overload
    def Parse(s: System.ReadOnlySpan[str]) -> System.Net.IPEndPoint:
        ...

    def Serialize(self) -> System.Net.SocketAddress:
        ...

    def ToString(self) -> str:
        ...

    @staticmethod
    @typing.overload
    def TryParse(s: str, result: typing.Optional[System.Net.IPEndPoint]) -> typing.Union[bool, System.Net.IPEndPoint]:
        ...

    @staticmethod
    @typing.overload
    def TryParse(s: System.ReadOnlySpan[str], result: typing.Optional[System.Net.IPEndPoint]) -> typing.Union[bool, System.Net.IPEndPoint]:
        ...


class TransportContext(System.Object, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def GetChannelBinding(self, kind: System.Security.Authentication.ExtendedProtection.ChannelBindingKind) -> System.Security.Authentication.ExtendedProtection.ChannelBinding:
        ...


class HttpStatusCode(System.Enum):
    """This class has no documentation."""

    Continue = 100

    SwitchingProtocols = 101

    Processing = 102

    EarlyHints = 103

    OK = 200

    Created = 201

    Accepted = 202

    NonAuthoritativeInformation = 203

    NoContent = 204

    ResetContent = 205

    PartialContent = 206

    MultiStatus = 207

    AlreadyReported = 208

    IMUsed = 226

    MultipleChoices = 300

    Ambiguous = 300

    MovedPermanently = 301

    Moved = 301

    Found = 302

    Redirect = 302

    SeeOther = 303

    RedirectMethod = 303

    NotModified = 304

    UseProxy = 305

    Unused = 306

    TemporaryRedirect = 307

    RedirectKeepVerb = 307

    PermanentRedirect = 308

    BadRequest = 400

    Unauthorized = 401

    PaymentRequired = 402

    Forbidden = 403

    NotFound = 404

    MethodNotAllowed = 405

    NotAcceptable = 406

    ProxyAuthenticationRequired = 407

    RequestTimeout = 408

    Conflict = 409

    Gone = 410

    LengthRequired = 411

    PreconditionFailed = 412

    RequestEntityTooLarge = 413

    RequestUriTooLong = 414

    UnsupportedMediaType = 415

    RequestedRangeNotSatisfiable = 416

    ExpectationFailed = 417

    MisdirectedRequest = 421

    UnprocessableEntity = 422

    Locked = 423

    FailedDependency = 424

    UpgradeRequired = 426

    PreconditionRequired = 428

    TooManyRequests = 429

    RequestHeaderFieldsTooLarge = 431

    UnavailableForLegalReasons = 451

    InternalServerError = 500

    NotImplemented = 501

    BadGateway = 502

    ServiceUnavailable = 503

    GatewayTimeout = 504

    HttpVersionNotSupported = 505

    VariantAlsoNegotiates = 506

    InsufficientStorage = 507

    LoopDetected = 508

    NotExtended = 510

    NetworkAuthenticationRequired = 511





import abc
import typing

import System
import System.Buffers
import System.Runtime.InteropServices

System_Buffers_StandardFormat = typing.Any

System_Buffers_ArrayPool_T = typing.TypeVar("System_Buffers_ArrayPool_T")
System_Buffers_MemoryManager_T = typing.TypeVar("System_Buffers_MemoryManager_T")
System_Buffers_IMemoryOwner_T = typing.TypeVar("System_Buffers_IMemoryOwner_T")


class OperationStatus(System.Enum):
    """
    This enum defines the various potential status that can be returned from Span-based operations
    that support processing of input contained in multiple discontiguous buffers.
    """

    Done = 0
    """The entire input buffer has been processed and the operation is complete."""

    DestinationTooSmall = 1
    """
    The input is partially processed, up to what could fit into the destination buffer.
    The caller can enlarge the destination buffer, slice the buffers appropriately, and retry.
    """

    NeedMoreData = 2
    """
    The input is partially processed, up to the last valid chunk of the input that could be consumed.
    The caller can stitch the remaining unprocessed input with more data, slice the buffers appropriately, and retry.
    """

    InvalidData = 3
    """
    The input contained invalid bytes which could not be processed. If the input is partially processed,
    the destination contains the partial result. This guarantees that no additional data appended to the input
    will make the invalid sequence valid.
    """


class IPinnable(metaclass=abc.ABCMeta):
    """Provides a mechanism for pinning and unpinning objects to prevent the GC from moving them."""

    def Pin(self, elementIndex: int) -> System.Buffers.MemoryHandle:
        """
        Call this method to indicate that the IPinnable object can not be moved by the garbage collector.
        The address of the pinned object can be taken.
        The offset to the element within the memory at which the returned  points to.
        
        :param elementIndex: The offset to the element within the memory at which the returned MemoryHandle points to.
        """
        ...

    def Unpin(self) -> None:
        """
        Call this method to indicate that the IPinnable object no longer needs to be pinned.
        The garbage collector is free to move the object now.
        """
        ...


class MemoryHandle(System.IDisposable):
    """A handle for the memory."""

    @property
    def Pointer(self) -> typing.Any:
        """Returns the pointer to memory, where the memory is assumed to be pinned and hence the address won't change."""
        ...

    def __init__(self, pointer: typing.Any, handle: System.Runtime.InteropServices.GCHandle = ..., pinnable: System.Buffers.IPinnable = ...) -> None:
        """
        Creates a new memory handle for the memory.
        
        :param pointer: pointer to memory
        :param handle: handle used to pin array buffers
        :param pinnable: reference to manually managed object, or default if there is no memory manager
        """
        ...

    def Dispose(self) -> None:
        """Frees the pinned handle and releases IPinnable."""
        ...


class ArrayPool(typing.Generic[System_Buffers_ArrayPool_T], System.Object, metaclass=abc.ABCMeta):
    """Provides a resource pool that enables reusing instances of arrays."""

    Shared: System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]
    """Retrieves a shared ArrayPool{T} instance."""

    @staticmethod
    @typing.overload
    def Create() -> System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]:
        """
        Creates a new ArrayPool{T} instance using default configuration options.
        
        :returns: A new ArrayPool{T} instance.
        """
        ...

    @staticmethod
    @typing.overload
    def Create(maxArrayLength: int, maxArraysPerBucket: int) -> System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]:
        """
        Creates a new ArrayPool{T} instance using custom configuration options.
        
        :param maxArrayLength: The maximum length of array instances that may be stored in the pool.
        :param maxArraysPerBucket: The maximum number of array instances that may be stored in each bucket in the pool.  The pool groups arrays of similar lengths into buckets for faster access.
        :returns: A new ArrayPool{T} instance with the specified configuration options.
        """
        ...

    def Rent(self, minimumLength: int) -> typing.List[System_Buffers_ArrayPool_T]:
        """
        Retrieves a buffer that is at least the requested length.
        
        :param minimumLength: The minimum length of the array needed.
        :returns: An array that is at least  in length.
        """
        ...

    def Return(self, array: typing.List[System_Buffers_ArrayPool_T], clearArray: bool = False) -> None:
        """
        Returns to the pool an array that was previously obtained via Rent on the same
        ArrayPool{T} instance.
        
        :param array: The buffer previously obtained from Rent to return to the pool.
        :param clearArray: If true and if the pool will store the buffer to enable subsequent reuse, Return will clear  of its contents so that a subsequent consumer via Rent will not see the previous consumer's content.  If false or if the pool will release the buffer, the array's contents are left unchanged.
        """
        ...


class IMemoryOwner(typing.Generic[System_Buffers_IMemoryOwner_T], System.IDisposable, metaclass=abc.ABCMeta):
    """Owner of MemoryT that is responsible for disposing the underlying memory appropriately."""

    @property
    @abc.abstractmethod
    def Memory(self) -> System.Memory[System_Buffers_IMemoryOwner_T]:
        """Returns a MemoryT."""
        ...


class MemoryManager(typing.Generic[System_Buffers_MemoryManager_T], System.Object, System.Buffers.IMemoryOwner[System_Buffers_MemoryManager_T], System.Buffers.IPinnable, metaclass=abc.ABCMeta):
    """Manager of System.Memory{T} that provides the implementation."""

    @property
    def Memory(self) -> System.Memory[System_Buffers_MemoryManager_T]:
        """Returns a System.Memory{T}."""
        ...

    @typing.overload
    def CreateMemory(self, length: int) -> System.Memory[System_Buffers_MemoryManager_T]:
        """
        Returns a System.Memory{T} for the current MemoryManager{T}.
        
        This method is protected.
        
        :param length: The element count in the memory, starting at offset 0.
        """
        ...

    @typing.overload
    def CreateMemory(self, start: int, length: int) -> System.Memory[System_Buffers_MemoryManager_T]:
        """
        Returns a System.Memory{T} for the current MemoryManager{T}.
        
        This method is protected.
        
        :param start: The offset to the element which the returned memory starts at.
        :param length: The element count in the memory, starting at element offset .
        """
        ...

    @typing.overload
    def Dispose(self) -> None:
        """Implements IDisposable."""
        ...

    @typing.overload
    def Dispose(self, disposing: bool) -> None:
        """
        Clean up of any leftover managed and unmanaged resources.
        
        This method is protected.
        """
        ...

    def GetSpan(self) -> System.Span[System_Buffers_MemoryManager_T]:
        """Returns a span wrapping the underlying memory."""
        ...

    def Pin(self, elementIndex: int = 0) -> System.Buffers.MemoryHandle:
        """
        Returns a handle to the memory that has been pinned and hence its address can be taken.
        
        :param elementIndex: The offset to the element within the memory at which the returned MemoryHandle points to. (default = 0)
        """
        ...

    def Unpin(self) -> None:
        """Lets the garbage collector know that the object is free to be moved now."""
        ...


class StandardFormat(System.IEquatable[System_Buffers_StandardFormat]):
    """
    Represents a standard formatting string without using an actual String. A StandardFormat consists of a character (such as 'G', 'D' or 'X')
    and an optional precision ranging from 0..99, or the special value NoPrecision.
    """

    NoPrecision: int = ...
    """Precision values for format that don't use a precision, or for when the precision is to be unspecified."""

    MaxPrecision: int = 99
    """The maximum valid precision value."""

    @property
    def Symbol(self) -> str:
        """The character component of the format."""
        ...

    @property
    def Precision(self) -> int:
        """The precision component of the format. Ranges from 0..9 or the special value NoPrecision."""
        ...

    @property
    def HasPrecision(self) -> bool:
        """true if Precision is a value other than NoPrecision"""
        ...

    @property
    def IsDefault(self) -> bool:
        """true if the StandardFormat == default(StandardFormat)"""
        ...

    FormatStringLength: int = 3
    """The exact buffer length required by Format."""

    def __init__(self, symbol: str, precision: int = ...) -> None:
        """
        Create a StandardFormat.
        
        :param symbol: A type-specific formatting character such as 'G', 'D' or 'X'
        :param precision: An optional precision ranging from 0..9 or the special value NoPrecision (the default)
        """
        ...

    @typing.overload
    def Equals(self, obj: typing.Any) -> bool:
        """Returns true if both the Symbol and Precision are equal."""
        ...

    @typing.overload
    def Equals(self, other: System.Buffers.StandardFormat) -> bool:
        """Returns true if both the Symbol and Precision are equal."""
        ...

    def GetHashCode(self) -> int:
        """Compute a hash code."""
        ...

    @staticmethod
    @typing.overload
    def Parse(format: System.ReadOnlySpan[str]) -> System.Buffers.StandardFormat:
        """Converts a ReadOnlySpan{Char} into a StandardFormat"""
        ...

    @staticmethod
    @typing.overload
    def Parse(format: str) -> System.Buffers.StandardFormat:
        """Converts a classic .NET format string into a StandardFormat"""
        ...

    def ToString(self) -> str:
        """Returns the format in classic .NET format."""
        ...

    @staticmethod
    def TryParse(format: System.ReadOnlySpan[str], result: typing.Optional[System.Buffers.StandardFormat]) -> typing.Union[bool, System.Buffers.StandardFormat]:
        """Tries to convert a ReadOnlySpan{Char} into a StandardFormat. A return value indicates whether the conversion succeeded or failed."""
        ...





import typing

import QuantConnect.Brokerages.Bitfinex.Messages
import System


class Wallet(System.Object):
    """Account wallet balance"""

    @property
    def Type(self) -> str:
        """Wallet name (exchange, margin, funding)"""
        ...

    @Type.setter
    def Type(self, value: str):
        """Wallet name (exchange, margin, funding)"""
        ...

    @property
    def Currency(self) -> str:
        """Currency (e.g. USD, ...)"""
        ...

    @Currency.setter
    def Currency(self, value: str):
        """Currency (e.g. USD, ...)"""
        ...

    @property
    def Balance(self) -> float:
        """Wallet balance"""
        ...

    @Balance.setter
    def Balance(self, value: float):
        """Wallet balance"""
        ...

    @property
    def UnsettledInterest(self) -> float:
        """Unsettled interest"""
        ...

    @UnsettledInterest.setter
    def UnsettledInterest(self, value: float):
        """Unsettled interest"""
        ...


class TradeExecutionUpdate(System.Object):
    """Trade execution event on the account."""

    @property
    def TradeId(self) -> int:
        """Trade database id"""
        ...

    @TradeId.setter
    def TradeId(self, value: int):
        """Trade database id"""
        ...

    @property
    def Symbol(self) -> str:
        """Symbol (tBTCUSD, …)"""
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        """Symbol (tBTCUSD, …)"""
        ...

    @property
    def MtsCreate(self) -> int:
        """Execution timestamp"""
        ...

    @MtsCreate.setter
    def MtsCreate(self, value: int):
        """Execution timestamp"""
        ...

    @property
    def OrderId(self) -> int:
        """Order id"""
        ...

    @OrderId.setter
    def OrderId(self, value: int):
        """Order id"""
        ...

    @property
    def ExecAmount(self) -> float:
        """Positive means buy, negative means sell"""
        ...

    @ExecAmount.setter
    def ExecAmount(self, value: float):
        """Positive means buy, negative means sell"""
        ...

    @property
    def ExecPrice(self) -> float:
        """Execution price"""
        ...

    @ExecPrice.setter
    def ExecPrice(self, value: float):
        """Execution price"""
        ...

    @property
    def OrderType(self) -> str:
        """Order type"""
        ...

    @OrderType.setter
    def OrderType(self, value: str):
        """Order type"""
        ...

    @property
    def OrderPrice(self) -> float:
        """Order price"""
        ...

    @OrderPrice.setter
    def OrderPrice(self, value: float):
        """Order price"""
        ...

    @property
    def Maker(self) -> int:
        """1 if true, -1 if false"""
        ...

    @Maker.setter
    def Maker(self, value: int):
        """1 if true, -1 if false"""
        ...

    @property
    def Fee(self) -> float:
        """Fee ('tu' only)"""
        ...

    @Fee.setter
    def Fee(self, value: float):
        """Fee ('tu' only)"""
        ...

    @property
    def FeeCurrency(self) -> str:
        """Fee currency ('tu' only)"""
        ...

    @FeeCurrency.setter
    def FeeCurrency(self, value: str):
        """Fee currency ('tu' only)"""
        ...


class Position(System.Object):
    """Bitfinex position"""

    @property
    def Symbol(self) -> str:
        """Pair (tBTCUSD, …)."""
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        """Pair (tBTCUSD, …)."""
        ...

    @property
    def Status(self) -> str:
        """Status (ACTIVE, CLOSED)."""
        ...

    @Status.setter
    def Status(self, value: str):
        """Status (ACTIVE, CLOSED)."""
        ...

    @property
    def Amount(self) -> float:
        """Size of the position. A positive value indicates a long position; a negative value indicates a short position."""
        ...

    @Amount.setter
    def Amount(self, value: float):
        """Size of the position. A positive value indicates a long position; a negative value indicates a short position."""
        ...

    @property
    def BasePrice(self) -> float:
        """Base price of the position. (Average traded price of the previous orders of the position)"""
        ...

    @BasePrice.setter
    def BasePrice(self, value: float):
        """Base price of the position. (Average traded price of the previous orders of the position)"""
        ...

    @property
    def MarginFunding(self) -> float:
        """The amount of funding being used for this position."""
        ...

    @MarginFunding.setter
    def MarginFunding(self, value: float):
        """The amount of funding being used for this position."""
        ...

    @property
    def MarginFundingType(self) -> int:
        """0 for daily, 1 for term."""
        ...

    @MarginFundingType.setter
    def MarginFundingType(self, value: int):
        """0 for daily, 1 for term."""
        ...

    @property
    def ProfitLoss(self) -> float:
        """Profit & Loss"""
        ...

    @ProfitLoss.setter
    def ProfitLoss(self, value: float):
        """Profit & Loss"""
        ...

    @property
    def ProfitLossPerc(self) -> float:
        """Profit & Loss Percentage"""
        ...

    @ProfitLossPerc.setter
    def ProfitLossPerc(self, value: float):
        """Profit & Loss Percentage"""
        ...

    @property
    def PriceLiq(self) -> float:
        """Liquidation price"""
        ...

    @PriceLiq.setter
    def PriceLiq(self, value: float):
        """Liquidation price"""
        ...

    @property
    def Leverage(self) -> float:
        """Leverage used for the position"""
        ...

    @Leverage.setter
    def Leverage(self, value: float):
        """Leverage used for the position"""
        ...

    @property
    def PositionId(self) -> int:
        """Position ID"""
        ...

    @PositionId.setter
    def PositionId(self, value: int):
        """Position ID"""
        ...

    @property
    def MtsCreate(self) -> int:
        """Millisecond timestamp of creation"""
        ...

    @MtsCreate.setter
    def MtsCreate(self, value: int):
        """Millisecond timestamp of creation"""
        ...

    @property
    def MtsUpdate(self) -> int:
        """Millisecond timestamp of update"""
        ...

    @MtsUpdate.setter
    def MtsUpdate(self, value: int):
        """Millisecond timestamp of update"""
        ...

    @property
    def Type(self) -> int:
        """Identifies the type of position, 0 = Margin position, 1 = Derivatives position"""
        ...

    @Type.setter
    def Type(self, value: int):
        """Identifies the type of position, 0 = Margin position, 1 = Derivatives position"""
        ...

    @property
    def Collateral(self) -> float:
        """The amount of collateral applied to the open position"""
        ...

    @Collateral.setter
    def Collateral(self, value: float):
        """The amount of collateral applied to the open position"""
        ...

    @property
    def CollateralMin(self) -> float:
        """The minimum amount of collateral required for the position"""
        ...

    @CollateralMin.setter
    def CollateralMin(self, value: float):
        """The minimum amount of collateral required for the position"""
        ...

    @property
    def Meta(self) -> System.Object:
        """Additional meta information about the position (JSON string)"""
        ...

    @Meta.setter
    def Meta(self, value: System.Object):
        """Additional meta information about the position (JSON string)"""
        ...


class Ticker(System.Object):
    """A high level overview of the state of the market for a specified pair"""

    @property
    def Bid(self) -> float:
        """Price of last highest bid"""
        ...

    @Bid.setter
    def Bid(self, value: float):
        """Price of last highest bid"""
        ...

    @property
    def BidSize(self) -> float:
        """Sum of the 25 highest bid sizes"""
        ...

    @BidSize.setter
    def BidSize(self, value: float):
        """Sum of the 25 highest bid sizes"""
        ...

    @property
    def Ask(self) -> float:
        """Price of last lowest ask"""
        ...

    @Ask.setter
    def Ask(self, value: float):
        """Price of last lowest ask"""
        ...

    @property
    def AskSize(self) -> float:
        """Sum of the 25 lowest ask sizes"""
        ...

    @AskSize.setter
    def AskSize(self, value: float):
        """Sum of the 25 lowest ask sizes"""
        ...

    @property
    def DailyChange(self) -> float:
        """Amount that the last price has changed since yesterday"""
        ...

    @DailyChange.setter
    def DailyChange(self, value: float):
        """Amount that the last price has changed since yesterday"""
        ...

    @property
    def DailyChangeRelative(self) -> float:
        """Relative price change since yesterday (*100 for percentage change)"""
        ...

    @DailyChangeRelative.setter
    def DailyChangeRelative(self, value: float):
        """Relative price change since yesterday (*100 for percentage change)"""
        ...

    @property
    def LastPrice(self) -> float:
        """Price of the last trade"""
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        """Price of the last trade"""
        ...

    @property
    def Volume(self) -> float:
        """Daily volume"""
        ...

    @Volume.setter
    def Volume(self, value: float):
        """Daily volume"""
        ...

    @property
    def High(self) -> float:
        """Daily high"""
        ...

    @High.setter
    def High(self, value: float):
        """Daily high"""
        ...

    @property
    def Low(self) -> float:
        """Daily low"""
        ...

    @Low.setter
    def Low(self, value: float):
        """Daily low"""
        ...


class BaseMessage(System.Object):
    """This class has no documentation."""

    @property
    def Event(self) -> str:
        ...

    @Event.setter
    def Event(self, value: str):
        ...


class ErrorMessage(QuantConnect.Brokerages.Bitfinex.Messages.BaseMessage):
    """This class has no documentation."""

    @property
    def Message(self) -> str:
        ...

    @Message.setter
    def Message(self, value: str):
        ...

    @property
    def Code(self) -> int:
        ...

    @Code.setter
    def Code(self, value: int):
        ...

    @property
    def Level(self) -> str:
        """10301 : Already subscribed"""
        ...


class ChannelSubscription(QuantConnect.Brokerages.Bitfinex.Messages.BaseMessage):
    """This class has no documentation."""

    @property
    def Channel(self) -> str:
        ...

    @Channel.setter
    def Channel(self, value: str):
        ...

    @property
    def ChannelId(self) -> int:
        ...

    @ChannelId.setter
    def ChannelId(self, value: int):
        ...

    @property
    def Symbol(self) -> str:
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        ...


class ChannelUnsubscribing(QuantConnect.Brokerages.Bitfinex.Messages.BaseMessage):
    """This class has no documentation."""

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def ChannelId(self) -> int:
        ...

    @ChannelId.setter
    def ChannelId(self, value: int):
        ...


class AuthResponseMessage(QuantConnect.Brokerages.Bitfinex.Messages.BaseMessage):
    """This class has no documentation."""

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...


class Candle(System.Object):
    """This class has no documentation."""

    @property
    def Timestamp(self) -> int:
        ...

    @Timestamp.setter
    def Timestamp(self, value: int):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Volume(self) -> float:
        ...

    @Volume.setter
    def Volume(self, value: float):
        ...

    @typing.overload
    def __init__(self) -> None:
        ...

    @typing.overload
    def __init__(self, msts: int, close: float) -> None:
        ...

    @typing.overload
    def __init__(self, entries: typing.List[System.Object]) -> None:
        ...


class OrderFlags(System.Object):
    """Bitfinex Order Flags"""

    Hidden: int = 64
    """The hidden order option ensures an order does not appear in the order book; thus does not influence other market participants."""

    Close: int = 512
    """Close position if position present."""

    ReduceOnly: int = 1024
    """Ensures that the executed order does not flip the opened position."""

    PostOnly: int = 4096
    """The post-only limit order option ensures the limit order will be added to the order book and not match with a pre-existing order."""

    Oco: int = 16384
    """
    The one cancels other order option allows you to place a pair of orders stipulating that if one order is executed fully or partially,
    then the other is automatically canceled.
    """

    NoVarRates: int = 524288
    """Excludes variable rate funding offers from matching against this order, if on margin"""


class Order(System.Object):
    """Bitfinex Order"""

    @property
    def Id(self) -> int:
        """Order ID"""
        ...

    @Id.setter
    def Id(self, value: int):
        """Order ID"""
        ...

    @property
    def GroupId(self) -> int:
        """Group ID"""
        ...

    @GroupId.setter
    def GroupId(self, value: int):
        """Group ID"""
        ...

    @property
    def ClientOrderId(self) -> int:
        """Client Order ID"""
        ...

    @ClientOrderId.setter
    def ClientOrderId(self, value: int):
        """Client Order ID"""
        ...

    @property
    def Symbol(self) -> str:
        """Pair (tBTCUSD, …)"""
        ...

    @Symbol.setter
    def Symbol(self, value: str):
        """Pair (tBTCUSD, …)"""
        ...

    @property
    def MtsCreate(self) -> int:
        """Millisecond timestamp of creation"""
        ...

    @MtsCreate.setter
    def MtsCreate(self, value: int):
        """Millisecond timestamp of creation"""
        ...

    @property
    def MtsUpdate(self) -> int:
        """Millisecond timestamp of update"""
        ...

    @MtsUpdate.setter
    def MtsUpdate(self, value: int):
        """Millisecond timestamp of update"""
        ...

    @property
    def Amount(self) -> float:
        """Positive means buy, negative means sell."""
        ...

    @Amount.setter
    def Amount(self, value: float):
        """Positive means buy, negative means sell."""
        ...

    @property
    def AmountOrig(self) -> float:
        """Original amount"""
        ...

    @AmountOrig.setter
    def AmountOrig(self, value: float):
        """Original amount"""
        ...

    @property
    def Type(self) -> str:
        """
        The type of the order:
        - LIMIT, MARKET, STOP, STOP LIMIT, TRAILING STOP,
        - EXCHANGE MARKET, EXCHANGE LIMIT, EXCHANGE STOP, EXCHANGE STOP LIMIT,
        - EXCHANGE TRAILING STOP, FOK, EXCHANGE FOK, IOC, EXCHANGE IOC.
        """
        ...

    @Type.setter
    def Type(self, value: str):
        """
        The type of the order:
        - LIMIT, MARKET, STOP, STOP LIMIT, TRAILING STOP,
        - EXCHANGE MARKET, EXCHANGE LIMIT, EXCHANGE STOP, EXCHANGE STOP LIMIT,
        - EXCHANGE TRAILING STOP, FOK, EXCHANGE FOK, IOC, EXCHANGE IOC.
        """
        ...

    @property
    def TypePrev(self) -> str:
        """Previous order type"""
        ...

    @TypePrev.setter
    def TypePrev(self, value: str):
        """Previous order type"""
        ...

    @property
    def Flags(self) -> int:
        """Active flags for order"""
        ...

    @Flags.setter
    def Flags(self, value: int):
        """Active flags for order"""
        ...

    @property
    def Status(self) -> str:
        """
        Order Status:
        - ACTIVE,
        - EXECUTED @ PRICE(AMOUNT) e.g. "EXECUTED @ 107.6(-0.2)",
        - PARTIALLY FILLED @ PRICE(AMOUNT),
        - INSUFFICIENT MARGIN was: PARTIALLY FILLED @ PRICE(AMOUNT),
        - CANCELED,
        - CANCELED was: PARTIALLY FILLED @ PRICE(AMOUNT),
        - RSN_DUST (amount is less than 0.00000001),
        - RSN_PAUSE (trading is paused / paused due to AMPL rebase event)
        """
        ...

    @Status.setter
    def Status(self, value: str):
        """
        Order Status:
        - ACTIVE,
        - EXECUTED @ PRICE(AMOUNT) e.g. "EXECUTED @ 107.6(-0.2)",
        - PARTIALLY FILLED @ PRICE(AMOUNT),
        - INSUFFICIENT MARGIN was: PARTIALLY FILLED @ PRICE(AMOUNT),
        - CANCELED,
        - CANCELED was: PARTIALLY FILLED @ PRICE(AMOUNT),
        - RSN_DUST (amount is less than 0.00000001),
        - RSN_PAUSE (trading is paused / paused due to AMPL rebase event)
        """
        ...

    @property
    def Price(self) -> float:
        """Price"""
        ...

    @Price.setter
    def Price(self, value: float):
        """Price"""
        ...

    @property
    def PriceAvg(self) -> float:
        """Average price"""
        ...

    @PriceAvg.setter
    def PriceAvg(self, value: float):
        """Average price"""
        ...

    @property
    def PriceTrailing(self) -> float:
        """The trailing price"""
        ...

    @PriceTrailing.setter
    def PriceTrailing(self, value: float):
        """The trailing price"""
        ...

    @property
    def PriceAuxLimit(self) -> float:
        """Auxiliary Limit price (for STOP LIMIT)"""
        ...

    @PriceAuxLimit.setter
    def PriceAuxLimit(self, value: float):
        """Auxiliary Limit price (for STOP LIMIT)"""
        ...

    @property
    def Hidden(self) -> int:
        """1 if Hidden, 0 if not hidden"""
        ...

    @Hidden.setter
    def Hidden(self, value: int):
        """1 if Hidden, 0 if not hidden"""
        ...

    @property
    def PlacedId(self) -> int:
        """If another order caused this order to be placed (OCO) this will be that other order's ID"""
        ...

    @PlacedId.setter
    def PlacedId(self, value: int):
        """If another order caused this order to be placed (OCO) this will be that other order's ID"""
        ...

    @property
    def IsExchange(self) -> bool:
        """If this is exchange order"""
        ...


